import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.DateFormat;

import javax.print.PrintService;
import javax.print.attribute.AttributeSet;
import javax.print.attribute.standard.JobName;
import javax.swing.*;

import jpdf.attribute.*;
import jpdf.gui.dialog.PrintDialog;
import jpdf.gui.dialog.PrintDialogPreferences;
import jpdf.gui.dialog.SetupContentDialog;
import jpdf.gui.dialog.factory.SetupContentPrintDialogFactory;
import jpdf.gui.preview2.PageLayout;
import jpdf.gui.preview2.PreviewPrintableDialog;
import jpdf.manage.PrintManager;
import jpdf.manage.PrintParameterHolder;
import jpdf.manage.ResourceAdministrator;
import jpdf.printer.TableComponentPrinter;


/**
 * Class SetupPreviewPrintFrame implements a frame which displays a table containing 
 * the 50 States of America, and contains buttons which implement Page Setup, Print 
 * Preview, and Print functionality.  These buttons allow the user to preview and print 
 * the displayed table.  
 *
 * <P>This class provides an example of how to use the JPDF to implement printing 
 * capabilities.
 */
public class SetupPreviewPrintFrame
  extends JFrame
{
  //-----------------------------------------------------------------------
  // Inner Classes.
  //-----------------------------------------------------------------------

  /**
   * Window Close Listener.  The Application is terminated when the frame is 
   * closed.
   */
  private static class WindowCloseListener 
    extends WindowAdapter
  {
    public void windowClosing(WindowEvent theEvent)
    {
      // Shutdown the JPDF Environment.
      PrintManager.shutdownEnvironment();
      
      // Exit the application.
      System.exit(0); 
    }
  }

  /**
   * The Page Setup button displays the Page Setup Dialog which allows the user to 
   * specify the Print Attributes which control the page that will be used to print 
   * the table.
   */  
  private class PageSetupButton
    extends JButton
    implements ActionListener
  {
    private PageSetupButton() 
    {
      super("Page Setup");
      
      addActionListener(this);
    }

    public void actionPerformed(ActionEvent theEvent)
    {
      SetupContentDialog theDialog = null;

      // Create and display the Page Setup Dialog.
      setWaitCursor(); 
      theDialog = SetupContentDialog.createTableComponentContentDialog(
                     SetupPreviewPrintFrame.this,  
                     PrintManager.getDefaultPrintAttributes());
      setDefaultCursor();
      theDialog.show(); 
    }  
  }

  /**
   * The Preview button displays the Preview Dialog which allows the 
   * user to preview the table.
   */
  private class PreviewButton
    extends JButton
    implements ActionListener
  {
    private PreviewButton() 
    {
      super("Preview");
      
      addActionListener(this);
    }

    public void actionPerformed(ActionEvent theEvent) 
    {
      PreviewPrintableDialog thePreviewDialog = null;
      
      // Create and display the Preview Dialog.
      // NOTE: The Dialog Factory creates the Page Setup and Print dialogs that can 
      //       be displayed from within the Preview Dialog.
      setWaitCursor(); 
      thePreviewDialog = new PreviewPrintableDialog(SetupPreviewPrintFrame.this, 
                                                    buildPrinter());
      thePreviewDialog.setDialogFactory(new SetupContentPrintDialogFactory()); 
      setDefaultCursor(); 
      thePreviewDialog.show(); 
    }
  }

  /**
   * Print Button.  Displays the Print Dialog which allows the user to specify the 
   * Print Attributes which control the Print Job that will be used to print the 
   * table, and to submit this Print Job.
   */
  private class PrintButton
    extends JButton
    implements ActionListener
  {
    private PrintButton() 
    {
      super("Print");
      
      addActionListener(this);
    }

    public void actionPerformed(ActionEvent theEvent) 
    {
      PrintService  thePrintService = null;
      PrintDialog   theDialog = null;

      // Create and display the Print Dialog.
      setWaitCursor(); 
      theDialog = new PrintDialog(SetupPreviewPrintFrame.this, 
                                  PrintManager.getDefaultPrintAttributes(), true);
      setDefaultCursor(); 
      theDialog.show(); 
      
      // Get the Selected Print Service.
      // NOTE: Null is returned if the user did not press the Print Button.
      thePrintService = theDialog.getSelectedPrintService();
      if (thePrintService != null) 
      {
        PrintManager.printPrintable(buildPrinter(), theDialog.getPrintInfoHolder());
      }
    }
  }

  
  //-----------------------------------------------------------------------
  // Class Initialization.
  //-----------------------------------------------------------------------
  static
  {
    // Set the Look-And-Feel.
    try
    {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    }
    catch (Exception theEx) {}  
  }
  
  
  //-----------------------------------------------------------------------
  // Class variables/constants.
  //-----------------------------------------------------------------------
  private static final String   TITLE = "The States of the Union";
  private static final int      PAGE_NOTE_FONT_SIZE = 12;
  
  //-----------------------------------------------------------------------
  // Instance variables/constants.
  //-----------------------------------------------------------------------
  private final UsStatesTable   USSTATES_TABLE = new UsStatesTable();
  
  
  //-----------------------------------------------------------------------
  // Constructors.
  //-----------------------------------------------------------------------
  
  private SetupPreviewPrintFrame() 
  {
    super(TITLE);

    // Initialize the frame contents.
    initializeContents();
  }

    
  //-----------------------------------------------------------------------
  // Public methods.
  //-----------------------------------------------------------------------

  /**
   * Application entry point.
   */
  public static void main(String theParams[]) 
  {
    // Initialize the JPDF Environment.
    initializeJpdfEnvironment();
    
    // Display the frame.  
    (new SetupPreviewPrintFrame()).setVisible(true);
  }      
   
  /**
   * Sets the cursor to the default cursor.
   */  
  public void setDefaultCursor()
  {
    setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    getGlassPane().setVisible(false);
  }

  /**
   * Sets the cursor to the wait cursor.
   */  
  public void setWaitCursor()
  {
    getGlassPane().setVisible(true);
    setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
  }

   
  //-----------------------------------------------------------------------
  // Private methods.
  //-----------------------------------------------------------------------

  /**
   * Builds the Table Component Printer to be used to print the table.
   */
  private TableComponentPrinter buildPrinter()
  {
    TableComponentPrinter thePrinter = null;
    
    // Build the Table Printer.
    thePrinter = new TableComponentPrinter(USSTATES_TABLE, TITLE, true, true);
    
    return thePrinter;
  }

  /**
   * Creates the Button Panel.
   */
  private JPanel createButtonPanel()
  {
    JPanel theButtonPanel = new JPanel();
    
    theButtonPanel.add(new PageSetupButton());
    theButtonPanel.add(new PreviewButton());
    theButtonPanel.add(new PrintButton());
    
    return theButtonPanel;
  }

  /**
   * Creates the print attribute used for the Page Footer.
   */
  private static PageFootDocument createPageFooterAttribute(Font thePageNoteFont)
  {
    String theText = null;
    String theFontFamiliy = thePageNoteFont.getFamily();

    theText = "This table contains information about the 50 states which comprise the " +
              "United States of America.  The US Mint is currently in the process of " +
              "issuing commemorative quarter dollar coins which honor every American " +
              "State.  These coins will be issued every 10 weeks through the year 2008 -- " +
              "in the order in which each state joined the Union.";
    
    return new PageFootDocument(theText, theFontFamiliy, PAGE_NOTE_FONT_SIZE, Color.BLUE, null);
  }
  
  /**
   * Creates the print attribute used for the Page Header.
   * @NOTE Special markers are used in the Page Header.  These are replaced by actual 
   *       values when the material is printed.
   */
  private static PageReportHeader createPageHeaderAttribute(Font thePageNoteFont)
  {
    PageReportHeader theReportHeader = null;
    String[]         theLeftLabels = new String[]{"System:", "Transaction:"};
    String[]         theLeftValues = new String[]{"UsInfoSys", "StateList"};
    String[]         theRightLabels = new String[]{"Date:", "Time:", "Page:"};
    String[]         theRightValues = new String[]
      {PrintParameterHolder.DATE_MARKER, PrintParameterHolder.TIME_MARKER, 
       PrintParameterHolder.PAGE_NUMBER_MARKER + " of " + PrintParameterHolder.PAGE_COUNT_MARKER};
    
    PageReportHeader.setDefaultTitleFont(thePageNoteFont.deriveFont(Font.BOLD, 16f));
    PageReportHeader.setDefaultLabelFont(thePageNoteFont.deriveFont((float)PAGE_NOTE_FONT_SIZE));
    PageReportHeader.setDefaultTextColor(Color.BLUE);
    PageReportHeader.setDefaultBackgroundColor(new Color(233,233,244));
    theReportHeader = new PageReportHeader(TITLE, theLeftLabels, theLeftValues, 
                                           theRightLabels, theRightValues);  

    return theReportHeader;
  }
    
  /**
   * Initializes the contents of this frame.
   */
  private void initializeContents()
  {
    Container theContentPane = getContentPane();
    
    // Configure the frame.
    setSize(600, 400);
    setLocationRelativeTo(null);

    // Add the contents.
    theContentPane.add((new JScrollPane(USSTATES_TABLE)), BorderLayout.CENTER);
    theContentPane.add(createButtonPanel(), BorderLayout.SOUTH);
    
    // Add listeners.
    addWindowListener(new WindowCloseListener());
  }
  
  /**
   * Initializes the JPDF environment.
   */
  private static void initializeJpdfEnvironment() 
  {
    AttributeSet theAttributeSet = null; 
    Font         thePageNoteFont = UIManager.getFont("Label.font");
    
    // Initialize the JPDF Environment.
    try
    {
      PrintManager.initializeEnvironment();
    }
    catch (Throwable theEx)
    {
      theEx.printStackTrace();
      System.exit(99); 
    }  
    
    // Preload the list of media supported by the Default Print Service on a background thread.  
    // This speeds up the initial display of the Page Setup dialog.
    PrintManager.preloadPrintServiceMediaOnThread();
    
    // Specify the Date Format used in Page Headers/Footers.
    ResourceAdministrator.setTimeStyle(DateFormat.SHORT);
    
    // Configure the Preview Dialog preferences.
    PreviewPrintableDialog.setToUsePreviousSizeLocation(true);
    PreviewPrintableDialog.setInitialPageLayout(PageLayout.SINGLE_COL);

    // Do not include the Banner Page Controller in the Print Dialogs.
    PrintDialogPreferences.setToIncludeBannerPageController(false);

    // Get the Default Attribute Set.
    theAttributeSet = PrintManager.getDefaultPrintAttributes(); 
    
    // Add the initial attributes we want to use for printing.
    // NOTE: Different types of attributes are used for the Header and Footer.  The Header is
    //       specified by a Report Header attribute.  The Footer is contained in a Text Document.
    theAttributeSet.add(new JobName("Preview Table Example", null));
    theAttributeSet.add(createPageHeaderAttribute(thePageNoteFont));
    theAttributeSet.add(createPageFooterAttribute(thePageNoteFont));
    theAttributeSet.add(new AlignmentFactorHorizontal(0.5f));
    theAttributeSet.add(new OffsetTop(0.1f, true));
    theAttributeSet.add(PageFooterOnFirst2LastPages.FIRST_PAGE_ONLY);
    theAttributeSet.add(PageFrame.TRUE);
    theAttributeSet.add(PageContentFrame.TRUE);
    theAttributeSet.add(UseBackgroundColor.TRUE);
    theAttributeSet.add(BackgroundColorOnPage.FALSE);
    theAttributeSet.add(ContentHeadersOnEveryPage.TRUE);
    theAttributeSet.add(HighlightSelections.TRUE);    
  }  
}