import java.awt.*;
import java.io.File; 
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Enumeration;

import javax.swing.*;
import javax.swing.event.TreeExpansionEvent; 
import javax.swing.event.TreeExpansionListener;
import javax.swing.tree.*; 


/**
 * Class JavaFileTree displays a directory tree, and the Java files in the directories.
 */
public class JavaFileTree 
  extends JTree 
{  
  //-----------------------------------------------------------------------
  // Inner Classes.
  //-----------------------------------------------------------------------

  /**
   * File Node Renderer.
   */
  protected static class FileNodeRenderer
    extends DefaultTreeCellRenderer 
  {    
    private Font JAVA_FILE_FONT;
    private Font FOLDER_FONT;
    
    protected FileNodeRenderer(JTree theTree)
    {
      super();
            
      FOLDER_FONT = theTree.getFont();
      JAVA_FILE_FONT = FOLDER_FONT.deriveFont(Font.BOLD);   
      setBackground(null);
      setBackgroundNonSelectionColor(null);
    }  
    
    public Component getTreeCellRendererComponent(JTree theTree, Object theNodeValue, boolean isSelected, 
                                                  boolean isExpanded, boolean isLeafNode, int theRowIndex, 
                                                  boolean isFocused)
    {
      Component theRendererComponent = null;      
      
      // NOTE: Need to set font first so that the preferred size is computed correctly.
      setFont(isLeafNode ? JAVA_FILE_FONT : FOLDER_FONT); 
      
      //  Get the Renderer Component.
      theRendererComponent = super.getTreeCellRendererComponent(theTree, theNodeValue, isSelected, isExpanded, 
                                                                isLeafNode, theRowIndex, isFocused);   
                                                                
      return theRendererComponent;
    }
  }
    
  /**
   * File Node.
   */
  protected static class FileNode
    extends DefaultMutableTreeNode 
  {    
    protected final   File    NODE_FILE;
    protected final   boolean IS_DIRECTORY_FILE;
    
    protected boolean populated = false;
 
    protected FileNode(File theNodeFile, Object theUserObject, boolean isDirectory)
    { 
      // NOTE: The string representation of the User Object is used as the label for this node.
      super(theUserObject);
      
      NODE_FILE = theNodeFile;
      IS_DIRECTORY_FILE = isDirectory; 
    } 
  
    /**
    * Gets the Child Nodes.  The child list is populated if not already populated.
    */
    public Enumeration children()
    {
      if (!populated) populateChildList();
      
      return super.children();
    }  
  
   /**
    * Determines if this node can have child nodes.  Returns true if the associated file is a directory
    * and has not yet been populated or has been populated and has children, else false.
    */
    public boolean getAllowsChildren() 
    {
      return IS_DIRECTORY_FILE && (!populated || (getChildCount() > 0));
    }
  
   /**
    * Determines if this node is a Leaf Node.  Returns true if the associated file is not a directory, 
    * else false.
    */
    public boolean isLeaf() 
    {
      return !IS_DIRECTORY_FILE; 
    } 
    
   /**
    * Populates the child list of this node with all of the Java Files and directories contained in the 
    * directory associated with this node.  The node is flagged as being populated.  If the node is already
    * populated, then nothing is done.
    */
    protected boolean populateChildList() 
    {
      String[] theFileNames = null;
      File     theChildNodeFile = null;
      boolean  isWithAddedNodes = false;
      
      // If the node is already populated then exit.
      if (populated)  return false;   
    
      // Get names of all of the directories and files in the directory associated with this node.
      theFileNames = NODE_FILE.list();
    
      try 
      {         
        // Process the Java files.
        for (int ii = 0; ii < theFileNames.length; ii++) 
        {
          theChildNodeFile = new File(NODE_FILE, theFileNames[ii]);
          if (!theChildNodeFile.isDirectory() && theFileNames[ii].endsWith(".java")) 
          {
            add(new FileNode(theChildNodeFile, theChildNodeFile.getName(), false));
            isWithAddedNodes = true;
          } 
        } 

        // Process the directories.
        for (int ii = 0; ii < theFileNames.length; ii++) 
        {
          theChildNodeFile = new File(NODE_FILE, theFileNames[ii]);
          if (theChildNodeFile.isDirectory()) 
          {
            add(new FileNode(theChildNodeFile, theChildNodeFile.getName(), true));
            isWithAddedNodes = true;
          } 
        } 
      }     
      // Ignore file access problems.
      catch (Throwable theEx) {}
      
      // Flag the node as being populated.
      populated = true; 
     
      return isWithAddedNodes; 
    }
  }  

  /**
   * Node Event Handler.
   */
  protected static class NodeEventHandler
    implements TreeExpansionListener 
  {
    public void treeExpanded(TreeExpansionEvent theEvent) 
    { 
      TreePath  thePath = theEvent.getPath();
      FileNode  theNode = (FileNode)thePath.getLastPathComponent();
      JTree     theTree = null;
    
      if (theNode.populateChildList())   
      {
        theTree = (JTree)theEvent.getSource(); 
        ((DefaultTreeModel)theTree.getModel()).nodeStructureChanged(theNode);
      } 
    } 

    public void treeCollapsed(TreeExpansionEvent theEvent) 
    {
    } 
  }

  
  //-----------------------------------------------------------------------
  // Class variables/constants.
  //-----------------------------------------------------------------------
  private static final Icon JAVA_FILE_ICON = new ImageIcon("javaCup.gif");
  
 
  //-----------------------------------------------------------------------
  // Constructors.
  //-----------------------------------------------------------------------
  
  public JavaFileTree(String theRootPath) 
    throws IOException 
  {
    super(createModel(theRootPath));
  
    // Initialize this tree.
    initialize(); 
  } 
    
  /**
   * Creates an instance of this class using the given Tree Model.
   * NOTE: The presence of this constructor allows the Tree Component Printer to 
   *       create a print clone without the need to instantiate a new Tree Model.
   */
  public JavaFileTree(TreeModel theModel) 
  {
    super(theModel);
  
    // Initialize this tree.
    initialize(); 
  } 
    
    
  //-----------------------------------------------------------------------
  // Private Methods.
  //-----------------------------------------------------------------------
  
  /**
   * Creates the Tree Model.
   */
  private static DefaultTreeModel createModel(String theRootPath) 
    throws IOException 
  {
    File     theRootFile = new File(theRootPath);                       
    FileNode theRootNode = null;

    // Determine if the Root File exists and whether it is a directory.
    if (!theRootFile.exists()) 
    {
      throw new FileNotFoundException("Directory " + theRootFile + " does not exist.");
    }    
    if (!theRootFile.isDirectory())
    {
      throw new IOException("Directory " + theRootFile + " is not a directory.");
    }  
    
    // Create the Root Node.
    theRootNode = new FileNode(theRootFile, theRootFile.getCanonicalPath(), true);
  
    // Populate the root node with its Java Files and subdirectories.
    theRootNode.populateChildList();   
    
    return new DefaultTreeModel(theRootNode); 
  } 
  
  /**
   * Initializes this tree.
   */
  private void initialize()
  {
    DefaultTreeCellRenderer theCellRenderer = null;
    int                     theRowHeight = getRowHeight();
    int                     theJavaIconHeight = 0;
    
    // Configure the tree.
    setBackground(new Color(222,255,222));
    setBorder(BorderFactory.createLineBorder(Color.ORANGE, 5));
    setFont(new Font("Sans Serif", Font.PLAIN, 12));
    
    // Set the Node Renderer.
    theCellRenderer = new FileNodeRenderer(this);
    setCellRenderer(theCellRenderer);   
    
    // Set the Leaf Node icon.  The leaf nodes are the Java files. 
    if (JAVA_FILE_ICON != null)
    {  
      theCellRenderer.setLeafIcon(JAVA_FILE_ICON);
      theJavaIconHeight = JAVA_FILE_ICON.getIconHeight();
      if ((theJavaIconHeight + 2) > theRowHeight) 
      {
        setRowHeight(theJavaIconHeight + 2);
      }
    }
  
    // Add listener for node expansion/collapse events.
    addTreeExpansionListener(new NodeEventHandler());
  }   
}  